  <!--
    Play Message Modal
  -->
  <div class="modal fade" id="play-message-modal" tabindex="-1" role="dialog" aria-labelledby="play-msg-label" aria-hidden="true">
    <div class="modal-dialog" role="document">
      <div class="modal-content">
        <div class="modal-header">
          <h5 class="modal-title" id="play-msg-label">Playing Message</h5>

          <button type="button" class="pause close" data-dismiss="modal" aria-label="Close">
            <span aria-hidden="true">&times;</span>
          </button>
        </div>
        <div class="modal-body">
          <h6 id="call-no">Voice message from call</h6>
          <audio id="audio" controls preload="metadata">
            <source type="audio/wav">
            Your browser does not support the audio element.
          </audio>
        </div>
        <div class="modal-footer">
          <button type="button" class="pause btn btn-secondary" data-dismiss="modal">Close</button>
        </div>
      </div>
    </div>
  </div>
